-- ローカルデータの管理
--
-- Satofumi KAMIMURA
-- $Id$

require("lsqlite3")
require("kimariji_group")


local DB_Name = "savedata/local.sqlite3"
local db_ = nil


-- テーブルを初期化
function db_local_create_table()
   return db_:exec[[
         CREATE TABLE history (datetime, start_sec, end_sec, miss_types);
         CREATE TABLE start_delay (waka_id, datetime, start_sec);
         CREATE TABLE kimariji_start_delay (group_id, datetime, start_sec);
   ]]
end


-- テーブルに初期データを格納
function db_local_initialize_table()

   -- 最大の入力遅延を、1 時間(最大値)で初期化する
   db_:exec("BEGIN TRANSACTION")

   -- 和歌の入力の開始時間、用のテーブル初期化
   for i = 1, 100 do
      local insert_stmt =
         db_:prepare("INSERT INTO start_delay VALUES(?, datetime('now'), ?)")
      insert_stmt:bind_values(i, (1000 * 60 * 60) + math.random())
      insert_stmt:step()
      insert_stmt:reset()
   end

   -- 決まり字グループ、用のテーブル初期化
   local group_id_max = kimariji_group_id_max()
   for i = 1, group_id_max do
      local insert_stmt =
         db_:prepare("INSERT INTO kimariji_start_delay VALUES(?, datetime('now'), ?)")
      insert_stmt:bind_values(i, (1000 * 60 * 60) + math.random())
      insert_stmt:step()
      insert_stmt:reset()
   end

   db_:exec("COMMIT")
end


-- DB に接続
function db_local_connect()

   db_ = sqlite3.open(DB_Name)
   if db_ == nil then
      -- エラーメッセージの表示
      -- !!!
      return false
   end

   -- DB を新規に作成した場合は、テーブルを作成する
   if db_local_create_table() == sqlite3.OK then
      db_local_initialize_table()
   end

   return true
end


function db_local_disconnect()
   if db_ == nil then
      return
   end

   db_:close()
end


-- 和歌の入力情報を登録
function db_local_register_waka(waka_id, start_sec)
   if db_ == nil then
      return
   end

   local update_stmt =
      db_:prepare("UPDATE start_delay SET datetime = datetime('now'), start_sec = ? WHERE waka_id = ?")
   update_stmt:bind_values(start_sec, waka_id)
   update_stmt:step()
   update_stmt:reset()
end


-- タイピング履歴の情報を登録
function db_local_register_history(start_sec, end_sec, miss_types)
   if db_ == nil then
      return
   end

   db_:exec[[
         INSERT INTO history VALUES (datetime('now'), start_sec, end_sec, miss_types);
   ]]
   local insert_stmt =
      db_:prepare("INSERT INTO history VALUES (datetime('now'), ?, ?, ?)")
   insert_stmt:bind_values(start_sec, end_sec, miss_types)
   insert_stmt:step()
   insert_stmt:reset()
end


-- 入力の開始時間が遅い順に和歌 ID を取得
function db_local_slow_waka(nth)
   if db_ == nil then
      return
   end

   local waka_id = {}
   local sql = "SELECT waka_id FROM start_delay ORDER BY start_sec DESC LIMIT " .. nth
   for row in db_:nrows(sql) do
      table.insert(waka_id, row.waka_id)
   end

   return waka_id
end


-- n 件のタイピング履歴を取得
function db_local_history(nth)
   if db_ == nil then
      return
   end

   local data = {}
   local sql = "SELECT start_sec, end_sec, miss_types FROM history WHERE start_sec is not null ORDER BY datetime DESC LIMIT " .. nth
   for row in db_:nrows(sql) do
      table.insert(data, {start_sec = row.start_sec,
                          end_sec = row.end_sec,
                          miss_types = row.miss_types})
   end

   local reversed_data = {}
   for i = 1, #data do
      table.insert(reversed_data, data[#data - i + 1])
   end

   return reversed_data
end


-- 選択した句以外の start_delay を増やす
function db_local_update_start_delay(id_array)
   if db_ == nil then
      return
   end

   local where_condition = ""
   for i = 1, #id_array do
      if i ~= 1 then
         where_condition = where_condition .. " and"
      end
      where_condition = where_condition .. " waka_id <> " .. id_array[i]
   end

   local sql = "update start_delay set start_sec = start_sec + 0.05 where" .. where_condition
   db_:exec(sql)
end


-- 選択した句以外の kimariji_start_delay を増やす
function db_local_update_kimariji_start_delay(group_id)
   if db_ == nil then
      return
   end

   local where_condition = ""
   where_condition = where_condition .. " group_id <> " .. group_id

   local sql = "update kimariji_start_delay set start_sec = start_sec + 0.05 where" .. where_condition
   db_:exec(sql)
end


-- 決まり字において、入力の開始時間が遅いグループ ID を返す
function db_local_slow_kimariji_waka()
   if db_ == nil then
      return
   end

   local sql = "SELECT group_id FROM kimariji_start_delay ORDER BY start_sec DESC LIMIT 1"
   for row in db_:nrows(sql) do
      return row.group_id
   end

   return 1
end


-- 決まり字において、入力の開始時間を登録する
function db_local_update_kimariji_start_delay(group_id)
   if db_ == nil then
      return
   end

   local sql = "update kimariji_start_delay set datetime = datetime('now'), start_sec = start_sec + 0.05 where group_id <> " .. group_id
   db_:exec(sql)
end
