-- 結果の表示 (練習用)
--
-- Satofumi KAMIMURA
-- $Id: display_result_practice.lua,v 47f83febdec5 2011/01/12 03:37:04 Satofumi $

require("db_local")


_G.Result_Shimonoku = 0
_G.Result_Kimariji = 1


-- state: 結果の表示 (練習用)
display_result_practice = {}


local NotInitialized = 0
local Initialized = 1
local DisplayGraph = 2

local start_sec_ = 0.0
local end_sec_ = 0.0
local miss_types_ = 0.0

local state_ = NotInitialized
local mode_ = Result_Shimonoku

local title_ = nil
local title_label_ = nil
local result_ = {}
local result_label_ = {}
local unit_ = {}
local unit_label_ = {}


function display_result_practice_clear()
   layer_:remove(title_label_)

   for i = 1, 3 do
      layer_:remove(result_label_[i])
      result_label_[i] = nil
      result_[i] = nil

      layer_:remove(unit_label_[i])
      unit_label_[i] = nil
      unit_[i] = nil
   end

   result_label_ = {}
   result_ = {}
   unit_label_ = {}
   unit_ = {}

   title_label_ = nil
   title_ = nil

   mode_ = Result_Shimonoku
end


-- 結果の出力モードを設定
function display_result_practice_set_mode(mode)
   mode_ = mode
end


-- 表示する結果のデータを登録
function display_result_practice_set_result(start_sec, end_sec, miss_types)
   start_sec_ = start_sec
   end_sec_ = end_sec
   miss_types_ = miss_types
end


display_result_practice["Enter"] =
   function(state)
      if state_ == NotInitialized then
         -- 表示する結果の作成

         -- タイトル
         font_:setPixelSize(40)
         title_ = TextSurface(font_, "今回の成績")
         title_label_ = Label(title_)
         title_label_:setPosition(Point(center(title_label_,
                                               screen_:rect().w / 2), title_y_))
         layer_:push_front(title_label_)

         -- 結果の表示
         local result_text = {
            "入力の開始時間     " .. string.format("%.1f", start_sec_),
            "入力の完了時間     " .. string.format("%.1f", end_sec_),
            "タイプミス数       " .. string.format("%.1f", miss_types_),
         }
         local unit_text = {
            " [秒/首]",
            " [秒/首]",
            " [回/首]",
         }

         if mode_ == Result_Kimariji then
            result_text = { result_text[1] }
            unit_text = { unit_text[1] }
         end

         local max_width = 0
         for i = 1, #result_text do
            font_:setPixelSize(34)
            result_[i] = TextSurface(font_, result_text[i])
            result_label_[i] = Label(result_[i])
            layer_:push_front(result_label_[i])

            font_:setPixelSize(23)
            unit_[i] = TextSurface(font_, unit_text[i])
            unit_label_[i] = Label(unit_[i])
            layer_:push_front(unit_label_[i])

            max_width = max(max_width,
                            result_label_[i]:rect().w + unit_label_[i]:rect().w)
         end

         -- 最も長い幅のラベルに合わせて中央揃えを行う
         local label_x = (screen_:rect().w - max_width) / 2
         local position = Point(label_x, title_y_ + 89)
         for i = 1, #result_text do
            result_label_[i]:setPosition(position)

            local unit_position =
               Point(position.x + result_label_[i]:rect().w + 8,
                     position.y + 8)
            unit_label_[i]:setPosition(unit_position)
            position.y = position.y + 54
         end

         local start_sec_data = typing_data_start_sec()
         if mode_ == Result_Shimonoku then
            -- 現状では、特になにもしない

         elseif mode_ == Result_Kimariji then
            -- 入力の遅かった和歌の決まり字と下の句の最初の２文字を表示する
            display_waka_kimariji_sample(start_sec_data, position.y)
         end

         -- 結果を記録する前に画面を描画してしまう
         redraw()

         if db_local_connect() then
            if mode_ == Result_Kimariji then
               db_local_update_kimariji_start_delay(group_id_, start_sec_)
            else
               for k, data in pairs(start_sec_data) do
                  -- 入力の開始時間、を更新する
                  db_local_register_waka(data.id, data.start_sec)
               end

               if mode_ == Result_Shimonoku then
                  -- タイピング履歴、用のデータを記録する
                  db_local_register_history(typing_data_average())
                  db_local_disconnect()
               end
            end
         end

         -- 終了の効果音を鳴らす
         sound_effect_epilogue_:play(0.0, 0.0, 0.1)

         -- ENTER の入力待ちへ
         require("wait_enter")
         state:stateMachine():changeState(wait_enter)

         state_ = Initialized
      end
   end


display_result_practice["Execute"] =
   function(state)
      redraw()

      if mode_ == Result_Kimariji then
         state_ = NotInitialized
         require("back_practice_menu")
         state:stateMachine():changeState(back_practice_menu)

      else
         if state_ ~= Initialized or escape_event_:isActive() then
            state_ = NotInitialized
            require("back_practice_menu")
            state:stateMachine():changeState(back_practice_menu)

         else
            state_ = DisplayGraph
            require("recent_history")
            state:stateMachine():changeState(recent_history)
         end
      end
      display_result_practice_clear()
   end


display_result_practice["Exit"] =
   function(state)
      collectgarbage("collect")
   end
