-- 和歌の表示
--
-- Satofumi KAMIMURA
-- $Id: display_waka.lua,v 6c3703ff864b 2010/05/26 01:07:02 Satofumi $
--
-- \todo かな入力のときも、BS, Enter, かな文字の入力候補を表示するかを検討する

require("waka_data")
require("kimariji_group")
require("waka_handler")


local InvalidId = -1
local WakaDrawHeight = 33
local WakaOffsetSize = 26
local LeftDrawSize = 28
local OffsetY = 12
local BorderMargin = 8
local BorderWidth = (WakaDrawSize * 11) + (BorderMargin * 2)
local BorderHeight = WakaDrawSize + (BorderMargin * 2)

local waka_text_ = {}
local waka_text_label_ = {}
local waka_yomibito_ = nil
local waka_yomibito_label_ = nil
local kimariji_bar_ = {}
local kimariji_bar_label_ = {}
local waka_left_ = nil
local waka_left_label_ = nil
local input_border_ = nil
local input_border_label_ = nil
local input_border_position_ = Point((screen_:rect().w - BorderWidth) / 2,
                                     screen_:rect().h - BorderHeight - 24)
local input_text_ = nil
local input_text_label_ = nil
local input_sample_ = nil
local input_sample_label_ = nil
local roman_sample_ = nil
local roman_sample_label_ = nil
local current_waka_id_ = InvalidId
local current_yomibito_id_ = InvalidId
local current_input_ku_ = InvalidId
local current_text_ = nil
local current_roman_text_ = nil
local current_sample_text_ = nil
local bs_position_ = nil
local cursor_label_draw_ = nil
local enter_position_ = nil

local scheduler_blink_cursor_id_ = nil
local scheduler_vibrate_bs_id_ = nil
local scheduler_vibrate_enter_id_ = nil
local wait_last_enter_ = false

local sound_effect_miss_ = SoundEffect(Sound_miss_)
local sound_effect_type_ = SoundEffect(Sound_type_)


-- カーソルの点滅用
function blinkCursor()
   while cursor_label_ ~= nil do
      cursor_label_draw_ = true
      Scheduler:wait(500)

      cursor_label_draw_ = false
      Scheduler:wait(500)
   end
end


-- BS の振動用
function display_waka_vibrate_bs()
   while scheduler_vibrate_bs_id_ ~= nil do
      bs_label_:setPosition(Point(bs_position_.x , bs_position_.y - 3))
      Scheduler:wait(100)

      bs_label_:setPosition(Point(bs_position_.x , bs_position_.y))
      Scheduler:wait(100)

      bs_label_:setPosition(Point(bs_position_.x , bs_position_.y - 3))
      Scheduler:wait(100)

      bs_label_:setPosition(Point(bs_position_.x , bs_position_.y))
      Scheduler:wait(400)
   end
end


function display_waka_vibrate_enter_check()
   while not wait_last_enter_ do
      Scheduler:yield()
   end
end


-- Enter の振動用
function display_waka_vibrate_enter()
   while true do
      display_waka_vibrate_enter_check()
      enter_label_:setPosition(Point(enter_position_.x + 0,
                                     enter_position_.y - 0))
      Scheduler:wait(500)

      display_waka_vibrate_enter_check()
      enter_label_:setPosition(Point(enter_position_.x + 0,
                                     enter_position_.y - 3))
      Scheduler:wait(100)

      display_waka_vibrate_enter_check()
      enter_label_:setPosition(Point(enter_position_.x + 0,
                                     enter_position_.y - 0))
      Scheduler:wait(100)

      display_waka_vibrate_enter_check()
      enter_label_:setPosition(Point(enter_position_.x + 0,
                                     enter_position_.y - 3))
      Scheduler:wait(100)
   end
end


-- 描画したコンポーネントのクリア
function display_waka_clear()
   display_waka_remove_inputs()

   waka_layer_:remove(waka_left_label_)
   waka_layer_:remove(waka_yomibito_label_)
   for i = 1, 5 do
      waka_layer_:remove(waka_text_label_[i])
      waka_text_label_[i] = nil
      waka_text_[i] = nil
   end
   waka_text_label_ = {}
   waka_text_ = {}

   for i = 1, #kimariji_bar_label_ do
      waka_layer_:remove(kimariji_bar_label_[i])
      kimariji_bar_label_[i] = nil
      kimariji_bar_[i] = nil
   end
   kimariji_bar_label_ = {}
   kimariji_bar_ = {}


   waka_left_label_ = nil
   waka_left_ = nil
   roman_sample_label_ = nil
   roman_sample_ = nil
   waka_yomibito_label_ = nil
   waka_yomibito_ = nil
   waka_text_label_ = {}

   scheduler_:remove(scheduler_blink_cursor_id_)
   scheduler_:remove(scheduler_vibrate_bs_id_)
   scheduler_:remove(scheduler_vibrate_enter_id_)
   scheduler_blink_cursor_id_ = nil
   scheduler_vibrate_bs_id_ = nil
   scheduler_vibrate_enter_id_ = nil
   wait_last_enter_ = false

   bs_position_ = nil

   current_waka_id_ = InvalidId
   current_yomibito_id_ = InvalidId
   current_input_ku_ = InvalidId
   current_text_ = nil
   current_roman_text_ = nil
   current_sample_text_ = nil

   input_border_label_ = nil
   input_border_ = nil
   input_text_ = nil
   input_text_label_ = nil
   input_sample_label_ = nil
   input_sample_ = nil
end


-- 決まり字のアンダーバーを描画
function display_waka_draw_kimariji_bar(i, font_size, sub_size)

   kimariji_bar_[i] = ColorSurface(Rect(0, 0, (font_size * sub_size) - 5, 4),
                                   Color(1.0, 0.0, 1.0, 1.0))
   kimariji_bar_label_[i] = Label(kimariji_bar_[i])
   local position = waka_text_label_[i]:position()
   position.x = position.x + 2
   position.y = position.y + font_size + OffsetY + 4
   kimariji_bar_label_[i]:setAlpha(0.0)
   kimariji_bar_label_[i]:setPosition(position)
   waka_layer_:push_front(kimariji_bar_label_[i])
end


-- [%] 指定で和歌を表示する
function display_waka(waka_id, percent, kimariji_mode)

   if current_waka_id_ ~= waka_id then
      current_waka_id_ = waka_id

      -- リソースの作りなおし
      for i = 1, 5 do
         waka_layer_:remove(waka_text_label_[i])
         waka_text_label_[i] = nil
         waka_text_[i] = nil
      end

      font_:setPixelSize(WakaDrawSize)
      for i = 1, 5 do
         local text = waka_data_[waka_id][i]
         if kimariji_mode then
            if i == 1 or (waka_kimariji_size_[waka_id] > 5 and i == 2) then
               text = waka_data_kana_[waka_id][i]
            end
         end
         local kana_text = waka_data_kana_[waka_id][i]
         waka_text_[i] = TextRubiFadeSurface(font_, text,
                                             rubi_font_, kana_text)
         waka_text_[i]:setFadePercent(0.0)
         waka_text_label_[i] = Label(waka_text_[i])
         waka_layer_:push_front(waka_text_label_[i])
      end

      -- 句の表示について、相対位置を設定
      local positions = {}
      positions[1] = Point(0, 0)
      positions[2] = Point(waka_text_label_[1]:rect().w + WakaOffsetSize, 0)
      positions[3] = Point(WakaDrawSize * 2, WakaDrawHeight * 2)
      positions[4] = Point(WakaDrawSize, WakaDrawHeight * 4)
      positions[5] = Point(waka_text_label_[4]:rect().w +
                           WakaDrawSize + WakaOffsetSize,
                           WakaDrawHeight * 4)

      -- 句の表示について、絶対位置を設定
      local waka_shimonoku_width =
         waka_text_label_[4]:rect().w + waka_text_label_[5]:rect().w
         + (WakaDrawSize * 2)
      local waka_kaminoku_width =
         waka_text_label_[1]:rect().w + waka_text_label_[2]:rect().w
         + WakaOffsetSize

      local waka_width = max(waka_shimonoku_width, waka_kaminoku_width)
      local x_offset = (screen_:rect().w - waka_width) / 2
      for i = 1, 5 do
         local position = Point(0, 0)
         position.x = positions[i].x + x_offset
         position.y = positions[i].y + OffsetY
         waka_text_label_[i]:setPosition(position)
      end

      if kimariji_mode then
         for i = 1, #kimariji_bar_label_ do
            waka_layer_:remove(kimariji_bar_label_[i])
            kimariji_bar_label_[i] = nil
            kimariji_bar_[i] = nil
         end

         local font_size = font_:pixelSize()
         local sub_size = min(waka_kimariji_size_[waka_id], 5)

         -- 決まり字の下部のラインを描画する
         display_waka_draw_kimariji_bar(1, font_size, sub_size)

         if waka_kimariji_size_[waka_id] > 5 then
            sub_size = waka_kimariji_size_[waka_id] - 5
            display_waka_draw_kimariji_bar(2, font_size, sub_size)
         end
      end
   end

   -- 句の描画
   local ku_ch = { 5, 7, 5, 7, 7 }
   local total_ch = 0
   for i = 1, 5 do
      total_ch = total_ch + ku_ch[i]
   end
   local current_ch = total_ch * percent / 100.0

   local ku_index = 5
   total_ch = 0
   for i = 1, 5 do
      if current_ch < ku_ch[i] then
         waka_text_[i]:setFadePercent(100 * (current_ch / ku_ch[i]))
      else
         waka_text_[i]:setFadePercent(100)
      end
      current_ch = current_ch - ku_ch[i]
      if current_ch < 0 then
         ku_index = i
         break
      end
   end

   if kimariji_mode then
      local bar_percent = min(percent * 5, 100)
      for i = 1, #kimariji_bar_label_ do
         kimariji_bar_label_[i]:setAlpha(bar_percent)
      end
   end

   return ku_index
end


-- [%] 指定で詠み人を表示する
function display_waka_yomibito(waka_id, percent)

   if current_yomibito_id_ ~= waka_id then
      current_yomibito_id_ = waka_id

      -- リソースの作りなおし
      waka_layer_:remove(waka_yomibito_label_)
      waka_yomibito_label_ = nil
      waka_yomibito_ = nil

      waka_yomibito_ = TextRubiFadeSurface(font_, yomibito_data_[waka_id],
                                           rubi_font_,
                                           yomibito_data_kana_[waka_id])
      waka_yomibito_:setFadePercent(0.0)
      waka_yomibito_label_ = Label(waka_yomibito_)

      waka_layer_:push_front(waka_yomibito_label_)

      local x_offset = screen_:rect().w - waka_yomibito_label_:rect().w
      waka_yomibito_label_:setPosition(Point(x_offset - WakaDrawSize,
                                             WakaDrawHeight * 6 + OffsetY * 3))
   end

   if percent >= 0 then
      waka_yomibito_:setFadePercent(percent);
   end
end


-- 残りの和歌数を表示
function display_waka_left(waka_index)
   waka_layer_:remove(waka_left_label_)
   waka_left_label_ = nil
   waka_left_ = nil

   local waka_left_size = #waka_id_array_ - waka_index
   font_:setPixelSize(LeftDrawSize)
   if waka_left_size > 0 then
      waka_left_ = TextSurface(font_, "残り" .. waka_left_size .. "首")
   else
      waka_left_ = TextSurface(font_, "入力完了")
   end

   waka_left_label_ = Label(waka_left_)
   local position = Point(12, bottom(waka_left_label_, screen_:rect().h - 12))
   waka_left_label_:setPosition(position)
   waka_layer_:push_front(waka_left_label_)
end


-- 残りの和歌数を消去
function display_waka_remove_left()
   waka_layer_:remove(waka_left_label_)
end


-- 入力サンプルの表示
function display_waka_input_sample(waka_id, input_ku, current_text,
                                   is_roman, view_ku)

   -- コルーチンの登録
   if scheduler_vibrate_bs_id_  == nil then
      scheduler_vibrate_bs_id_ = scheduler_:insert(display_waka_vibrate_bs)
      bs_position_ = Point(0, 0)
   end
   if scheduler_vibrate_enter_id_  == nil then
      scheduler_vibrate_enter_id_ =
         scheduler_:insert(display_waka_vibrate_enter)
   end

   -- 入力するべき文字列の生成
   if view_ku >= input_ku + 4 then
      local input_sample_text =
         waka_handler_convert_old_kana(waka_data_kana_[waka_id][input_ku + 4])
      if current_sample_text_ ~= input_sample_text then
         current_sample_text_ = input_sample_text

         -- リソースの作りなおし
         waka_layer_:remove(input_sample_label_)
         input_sample_ = TextSurface(font_, input_sample_text)
         input_sample_label_ = Label(input_sample_)

         local position = Point(input_border_position_.x + BorderMargin,
                                input_border_position_.y - WakaDrawSize)
         input_sample_label_:setPosition(position)

         waka_layer_:push_front(input_sample_label_)
      end
   end
   if view_ku < input_ku + 4 then
      waka_layer_:remove(input_sample_label_)
   end

   -- ローマ字の入力候補を作成
   local center_x = screen_:rect().w / 2
   local position = Point(center_x, 305)
   if current_roman_text_ ~= current_text then
      current_roman_text_ = current_text

      local sample_text =
         waka_handler_convert_old_kana(waka_data_kana_[waka_id][input_ku + 4])
      local miss_detected = false
      sample_text, miss_detected =
         waka_handler_input_sample(sample_text, current_text)

      waka_layer_:remove(roman_sample_label_)
      if sample_text:len() > 0 then
         roman_sample_label_ = nil
         roman_sample_ = nil

         roman_sample_ = TextSurface(font_, sample_text)
         roman_sample_label_ = Label(roman_sample_)

         position.x = center(roman_sample_label_, center_x)
         roman_sample_label_:setPosition(position)
      else
         roman_sample_label_ = nil
      end

      -- 入力が間違っている場合に BS を表示
      waka_layer_:remove(bs_label_)
      if miss_detected then
         if is_roman then
            bs_position_ = Point(position.x - bs_label_:rect().w - 4,
                                 position.y + 11)
         else
            bs_position_ = Point(center_x - bs_label_:rect().w - 4,
                                 position.y + 11)
         end
         bs_label_:setPosition(bs_position_)
         waka_layer_:push_front(bs_label_)
      end

      -- タイプの効果音を鳴らす
      if miss_detected then
         sound_effect_miss_:play(0.0, 0.0, 0.1)
      else
         if current_text:len() > 0 then
            sound_effect_type_:play(0.0, 0.0, 0.1)
         end
      end

      -- 改行の画像を表示
      if sample_text:len() > 0 or miss_detected then
         if sample_text:len() > 0 then
            position.x = position.x + roman_sample_label_:rect().w
         end
         wait_last_enter_ = false
      else
         wait_last_enter_ = true
      end

      position.x = position.x + 4
      position.y = position.y + 10
      if is_roman then
         enter_position_ = position
      else
         enter_position_ = Point(center_x, position.y)
      end

      enter_label_:setPosition(enter_position_)
      if is_roman or wait_last_enter_ then
         waka_layer_:push_front(enter_label_)
      end
   end

   if view_ku >= input_ku + 4 and roman_sample_label_ ~= nil then
      if is_roman then
         waka_layer_:push_front(roman_sample_label_)
      end
   end
end


-- ローマ字の入力サンプル、入力欄を消す
function display_waka_remove_inputs()

   waka_layer_:remove(bs_label_)
   waka_layer_:remove(enter_label_)
   waka_layer_:remove(cursor_label_)
   waka_layer_:remove(input_text_label_)
   waka_layer_:remove(input_border_label_)
   waka_layer_:remove(input_sample_label_)
   waka_layer_:remove(roman_sample_label_)

   input_sample_label_ = nil
   input_border_label_ = nil
   input_border_ = nil

   scheduler_:remove(scheduler_blink_cursor_id_)
   scheduler_:remove(scheduler_vibrate_enter_id_)
   scheduler_blink_cursor_id_ = nil
   scheduler_vibrate_enter_id_ = nil
end


-- 入力中の文字列の表示
function display_waka_input(text)

   if input_border_label_ == nil then
      input_border_ = ColorSurface(Rect(0, 0, BorderWidth, BorderHeight),
                                   Color(0.0, 0.0, 0.0, 1.0))
      input_border_label_ = Label(input_border_)
      input_border_label_:setPosition(input_border_position_)

      scheduler_blink_cursor_id_ = scheduler_:insert(blinkCursor)
   end

   local position = Point(input_border_position_.x + BorderMargin,
                          input_border_position_.y + BorderMargin + 3)

   -- 内容が変更されていたら、リソースを作りなおす
   if current_text_ ~= text then
      current_text_ = text

      -- 表示
      waka_layer_:remove(input_text_label_)
      waka_layer_:push_front(input_border_label_)

      input_text_label_ = nil
      input_text_ = nil

      if #text > 0 then
         -- 入力テキストの表示
         font_:setForegroundColor(Color(0.9, 0.9, 0.9, 1.0))
         font_:setBackgroundColor(Color(0.3, 0.3, 0.3, 1.0))
         input_text_ = TextSurface(font_, text)
         font_:setForegroundColor(Color(0.0, 0.0, 0.0, 1.0))
         font_:setBackgroundColor(Color(0.7, 0.7, 0.7, 1.0))
         input_text_label_ = Label(input_text_)
         input_text_label_:setPosition(position)
         waka_layer_:push_front(input_text_label_)
      end
   end

   if input_text_label_ ~= nil then
      position.x = position.x + input_text_label_:rect().w
   end

   position.x = position.x + 2
   cursor_label_:setPosition(position)
   waka_layer_:remove(cursor_label_)
   if cursor_label_draw_ then
      waka_layer_:push_front(cursor_label_)
   end
end


-- 決まり字の文字列までを表示する
function display_waka_kimariji(waka_id)

   -- !!!
end


-- 決まり字用に、下の句の最初の２文字を表示する
function display_waka_kimariji_input_sample(kimariji_input_sample, view_ku)

   if view_ku < 4 then
      return
   end

   -- 下の句の最初の２文字を描画する
   if input_sample_label_ == nil then
      waka_layer_:remove(input_sample_label_)
      input_sample_ = TextSurface(font_, kimariji_input_sample)
      input_sample_label_ = Label(input_sample_)

      local position = Point(input_border_position_.x + BorderMargin,
                             input_border_position_.y - WakaDrawSize)
      input_sample_label_:setPosition(position)
      waka_layer_:push_front(input_sample_label_)
   end
end


-- 決まり字の表を描画する
function display_waka_kimariji_sample(start_sec_data, position_y)

   -- 入力の遅かった和歌の決まり字と下の句の最初の２文字を表示する
   -- !!!
end