-- 下の句タイピング
--
-- Satofumi KAMIMURA
-- $Id: shimonoku_typing.lua,v 3d779acf5d87 2010/05/22 15:22:51 Satofumi $

-- \todo 和歌の入力後、Enter が押されたら waka_layer_ をフェードアウトさせる

require("input_handler")
require("display_waka")
require("waka_handler")
require("typing_data")
require("display_result_practice")


-- state: 下の句タイピング
shimonoku_typing = {}


local DisplaySubTitle = 0
local DisplayWaka = 1
local DisplayResult = 2

local state_ = DisplaySubTitle
local waka_index_ = 0
local waka_id_ = 1
local input_ku_ = 0
local display_percent_ = 0
local stop_watch_ = nil
local start_sec_ = 0.0
local converted_text_ = ""
local input_completed_ = false

local sound_effect_prologue_ = nil


function shimonoku_typing_handle_escape(state)

   -- 入力文字があるときの ESC では、入力文字をクリアするだけにする
   if #converted_text_ > 0 then
      converted_text_ = ""
      typing_data_add(TypingData_ESC, stop_watch_:ticks())
   else
      require("practice_menu")
      back_music_:stop(300)
      state:stateMachine():changeState(practice_menu)
      shimonoku_typing_clearState()
   end
end


function shimonoku_typing_clearState()
   display_waka_clear()
   display_result_practice_clear()
   waka_layer_:clear()

   state_ = 0
   waka_index_ = 0
   input_ku_ = 0

   stop_watch_ = nil
end


shimonoku_typing["Enter"] =
   function(state)
      if state_ == DisplaySubTitle then
         -- サブタイトルの作成
         state:stateMachine():changeState(display_subtitle)
         state_ = DisplayWaka

         -- 曲の再生を停止する
         back_music_:stop(300)

         -- サブタイトル用の曲を再生
         sound_effect_prologue_ = SoundEffect(Sound_prologue_)
         sound_effect_prologue_:play(0.0, 0.0, 0.1)

      elseif state_ == DisplayWaka then
         if sound_effect_prologue_ ~= nil then
            sound_effect_prologue_:stop()
         end
         text_input_:clear()
         display_percent_ = 0
         if waka_index_ == 0 then
            typing_data_clear(TypingData_Shimonoku)
            stop_watch_ = StopWatch()

            -- 曲の再生を開始する
            back_music_:play(Music_typing_, 100, BackMusic_Infinity, 100)
         end

         -- 入力処理の開始
         start_sec_ = 0.0
         waka_index_ = waka_index_ + 1
         input_ku_ = 0
         converted_text_ = ""
         bs_event_:isActive()
         display_waka_remove_left()

         if waka_index_ > #waka_id_array_ then
            state_ = DisplayResult
         else
            waka_id_ = waka_id_array_[waka_index_]
            input_completed_ = false
            typing_data_set_waka_id(waka_id_)

            display_waka(waka_id_, 0, false)
            display_waka_yomibito(waka_id_, 0)
         end
         stop_watch_:stop()
         stop_watch_:start()
      end
   end


shimonoku_typing["Execute"] =
   function(state)
      scheduler_:execute()
      redraw()

      if state_ == DisplayWaka then
         -- 和歌の入力中
         display_percent_ = 100.0 * stop_watch_:ticks() / YomiageSecond

         --if false then
         if not input_completed_ then
            -- 和歌の入力が完了していない
            local view_ku = display_waka(waka_id_, display_percent_, false)
            local yomibito_percent = (display_percent_ - 100) * 5
            display_waka_yomibito(waka_id_, yomibito_percent)

            if input_ku_ == 0 and #converted_text_ == 0 then
               start_sec_ = stop_watch_:ticks()
            end

            -- 入力文字列の変換
            local input = text_input_:text()
            text_input_:clear()
            typing_data_add_string(input, stop_watch_:ticks())
            local bs_pressed = false
            converted_text_, bs_pressed =
               input_handler_convert(converted_text_, input)
            if bs_pressed then
               typing_data_add(TypingData_BS, stop_watch_:ticks())
            end

            -- 入力フォームの表示
            display_waka_input(converted_text_)

            -- 入力候補を表示する
            display_waka_input_sample(waka_id_, input_ku_,
                                      converted_text_, is_roman_, view_ku)

            if enter_event_:isActive() then
               typing_data_add(TypingData_Enter, stop_watch_:ticks())

               -- 入力文字列の評価
               if waka_handler_input_is_completed(waka_id_, input_ku_,
                                                  converted_text_) then
                  -- 変換結果が一致
                  converted_text_ = ""
                  input_ku_ = input_ku_ + 1

                  if input_ku_ > 1 then
                     -- 首の入力が完了
                     input_ku_ = 0
                     input_completed_ = true
                     typing_data_lf(start_sec_ / 1000.0,
                                    stop_watch_:ticks() / 1000.0)

                     -- 入力フォームを消す
                     display_waka_remove_inputs()
                  end
               end
            end
         else
            -- 入力が完了した
            display_waka_left(waka_index_)
            display_waka(waka_id_, 100, false)
            display_waka_yomibito(waka_id_, 100)

            if waka_index_ >= #waka_id_array_ then
               -- 全ての和歌を入力したら、曲の再生を止める
               back_music_:stop(1000)
            end

            -- 入力完了の効果音を鳴らす
            sound_effect_input_terminated_:play(0.0, 0.0, 0.1)

            -- 改行を待つ
            require("wait_enter")
            state:stateMachine():changeState(wait_enter)
         end

      elseif escape_event_:isActive() then
         -- wait_enter 中に ESC が押されたときは、結果を表示せずに戻るように
         -- ESC の押下を評価する
         shimonoku_typing_handle_escape(state)

      elseif state_ == DisplayResult then
         shimonoku_typing_clearState()

         -- 結果の表示
         display_result_practice_set_result(typing_data_average())
         state:stateMachine():changeState(display_result_practice)
      end

      if escape_event_:isActive() then
         shimonoku_typing_handle_escape(state)
      end
   end


shimonoku_typing["Exit"] =
   function(state)
      sound_effect_prologue_ = nil

      collectgarbage("collect")
   end
