-- 初期化
--
-- Satofumi KAMIMURA
-- $Id: startup.lua,v e604d36a482a 2011/05/07 08:22:27 satofumi $
--
-- \todo 画像、フォントのリソースが読み込めないときは、エラーログを表示して終了するようにする
-- \todo 左クリックが Enter イベントとして登録されないのを修正する

require("strict")
require("common")
require("Scheduler")


_G.YomiageSecond = 12000.0      -- [msec]

_G.Music_menu_ = "resources/music_menu.mp3"
_G.Music_typing_ = "resources/music_typing.mp3"

_G.Sound_prologue_ = "resources/prologue.wav"
_G.Sound_epilogue_ = "resources/epilogue.wav"
_G.Sound_input_terminated_ = "resources/input_terminated.wav"
_G.Sound_miss_ = "resources/miss.wav"
_G.Sound_type_ = "resources/type.wav"

_G.sound_effect_epilogue_ = SoundEffect(Sound_epilogue_)
_G.sound_effect_input_terminated_ = SoundEffect(Sound_input_terminated_)

_G.screen_ = nil
_G.audio_ = nil
_G.back_music_ = nil
_G.layer_manager_ = nil
_G.layer_ = nil
_G.waka_layer_ = nil
_G.back_ = nil
_G.back_fill_ = nil
_G.back_label_ = nil
_G.bs_ = nil
_G.bs_label_ = nil
_G.enter_ = nil
_G.enter_label_ = nil
_G.cursor_ = nil
_G.cursor_label_ = nil
_G.font_ = nil
_G.rubi_font_ = nil
_G.cycle_timer_ = nil
_G.escape_event_ = nil
_G.enter_event_ = nil
_G.bs_event_ = nil
_G.alt_enter_event_ = nil
_G.left_cursor_event_ = nil
_G.right_cursor_event_ = nil
_G.up_key_event_ = nil
_G.down_key_event_ = nil
_G.quit_event_ = nil
_G.text_input_ = nil
_G.scheduler_ = nil

_G.WakaDrawSize = 37
_G.title_y_ = 70
_G.menu_y_ = 160

_G.waka_id_size_ = 5
_G.waka_id_array_ = {}
_G.waka_kimariji_size_ = {}
_G.group_id_ = 1

_G.account_ = nil
_G.key_code_ = nil


function checkQuit()
   if quit_event_:isActive() then
      -- ウィンドウサイズを元に戻してから終了させる
      screen_:setFullscreen(false)
      os.exit(0)
   end
end


function checkToggleScreen()
   if alt_enter_event_:isActive() then
      screen_:toggleScreen()
   end
end


function cleanupInput()
   left_cursor_event_:isActive()
   right_cursor_event_:isActive()
end


function expandMenu(menu)
   -- カーソルの左で一番上、右で一番下の項目を選択する
   if left_cursor_event_:isActive() then
      menu:setIndex(0)
   end
   if right_cursor_event_:isActive() then
      local last_index = menu:indexSize()
      menu:setIndex(last_index - 1)
   end

   if up_key_event_:isActive() then
      local current_index = menu:index()
      if current_index > 0 then
         menu:setIndex(current_index - 1)
      end
   end
   if down_key_event_:isActive() then
      local current_index = menu:index()
      local last_index = menu:indexSize()
      if current_index < (last_index - 1) then
         menu:setIndex(current_index + 1)
      end
   end

   -- ctrl-m, ctrl-j で選択を確定する
   if enter_event_:isActive() then
      menu:press(menu:index() + 1)
   end
end


function redraw()
   layer_manager_:updateEvent()
   screen_:clear()
   layer_manager_:draw()
   delay(cycle_timer_:waitMsec())
   layer_manager_:swap()
   checkQuit()
   checkToggleScreen()
end


-- state: 開始
startup = {}


startup["Enter"] =
   function(state)
      -- ガベージコレクションを停止
      collectgarbage("stop")

      -- 乱数を初期化
      math.randomseed(os.time())

      -- スクリーン設定を反映
      screen_ = Screen()
      if (not screen_:isFullscreen()) and is_fullscreen_ then
         screen_:toggleScreen()
      end

      -- 音源まわりの初期化
      audio_ = Audio()
      back_music_ = BackMusic()

      -- 音量設定を反映
      back_music_:setVolume(volume_music_index_ * 10)
      sound_effect_input_terminated_:setMasterVolume(volume_se_index_ * 10)

      -- 共通リソースの初期化
      layer_manager_ = LayerManager()
      layer_ = Layer()
      layer_:setAlpha(0.0)
      layer_manager_:push_front(layer_)
      waka_layer_ = Layer()
      layer_manager_:push_front(waka_layer_)
      font_ = Font("resources/font.ttf", 20, true)
      if not font_:isValid() then
         -- フォントがなければ終了する
         os.exit()
      end
      font_:setForegroundColor(Color(0.0, 0.0, 0.0, 1.0))
      font_:setBackgroundColor(Color(0.7, 0.7, 0.7, 1.0))

      rubi_font_ = Font("resources/font.ttf", 18, true)
      rubi_font_:setForegroundColor(Color(0.0, 0.0, 0.0, 1.0))
      rubi_font_:setBackgroundColor(Color(0.7, 0.7, 0.7, 1.0))

      cycle_timer_ = CycleTimer()

      -- 背景の塗りつぶし
      back_ = ImageSurface("resources/back.png", false)
      back_fill_ = FillSurface(screen_:rect(), back_)
      back_label_ = Label(back_fill_)
      layer_:push_front(back_label_)

      -- BS, Enter, Cursor 画像の生成
      bs_ = ImageSurface("resources/bs.png", true)
      bs_label_ = Label(bs_)
      enter_ = ImageSurface("resources/enter.png", true)
      enter_label_ = Label(enter_)
      cursor_ = ColorSurface(Rect(0, 0, 14, WakaDrawSize - 6),
                             Color(1.0, 1.0, 1.0, 1.0))
      cursor_label_ = Label(cursor_)

      if not back_:isValid() or not bs_:isValid() or not enter_:isValid() then
         -- 画像リソースがなければ終了する
         os.exit()
      end

      -- ESC キーの登録
      escape_event_ = CallbackEvent()
      escape_event_:key(SDLK_ESCAPE, SDL_KEYDOWN, KMOD_NONE, 0)
      layer_manager_:insertEvent(escape_event_)

      -- Enter キーの登録
      enter_event_ = CallbackEvent()
      enter_event_:key(SDLK_RETURN, SDL_KEYUP, KMOD_NONE, 0)
      enter_event_:key(SDLK_RETURN, SDL_KEYUP, KMOD_LCTRL, 0)
      enter_event_:key(SDLK_RETURN, SDL_KEYUP, KMOD_RCTRL, 0)
      enter_event_:key(SDLK_j, SDL_KEYUP, KMOD_LCTRL, 0)
      enter_event_:key(SDLK_j, SDL_KEYUP, KMOD_RCTRL, 0)
      enter_event_:key(SDLK_m, SDL_KEYUP, KMOD_LCTRL, 0)
      enter_event_:key(SDLK_m, SDL_KEYUP, KMOD_RCTRL, 0)

      -- !!! クリックが登録されない理由を確認して修正する
      --enter_event_:click(SDL_BUTTON_LEFT, SDL_MOUSEBUTTONDOWN);

      layer_manager_:insertEvent(enter_event_)

      -- BS キーの登録
      bs_event_ = CallbackEvent()
      bs_event_:key(SDLK_BACKSPACE, SDL_KEYDOWN, KMOD_NONE, 0)
      bs_event_:key(SDLK_h, SDL_KEYDOWN, KMOD_LCTRL, 0)
      bs_event_:key(SDLK_h, SDL_KEYDOWN, KMOD_RCTRL, 0)
      layer_manager_:insertEvent(bs_event_)

      -- ALT + Enter キーの登録
      alt_enter_event_ = CallbackEvent()
      alt_enter_event_:key(SDLK_RETURN, SDL_KEYDOWN, KMOD_LALT, 0)
      alt_enter_event_:key(SDLK_RETURN, SDL_KEYDOWN, KMOD_RALT, 0)
      layer_manager_:insertEvent(alt_enter_event_)

      -- 左右のカーソルキーの登録
      left_cursor_event_ = CallbackEvent()
      left_cursor_event_:key(SDLK_LEFT, SDL_KEYDOWN, KMOD_NONE, 0)
      left_cursor_event_:key(SDLK_b, SDL_KEYDOWN, KMOD_RCTRL, 0)
      left_cursor_event_:key(SDLK_b, SDL_KEYDOWN, KMOD_LCTRL, 0)
      layer_manager_:insertEvent(left_cursor_event_)

      right_cursor_event_ = CallbackEvent()
      right_cursor_event_:key(SDLK_RIGHT, SDL_KEYDOWN, KMOD_NONE, 0)
      right_cursor_event_:key(SDLK_f, SDL_KEYDOWN, KMOD_RCTRL, 0)
      right_cursor_event_:key(SDLK_f, SDL_KEYDOWN, KMOD_LCTRL, 0)
      layer_manager_:insertEvent(right_cursor_event_)

      -- 上下のキーの登録
      up_key_event_ = CallbackEvent()
      up_key_event_:key(SDLK_p, SDL_KEYDOWN, KMOD_LCTRL, 0)
      up_key_event_:key(SDLK_p, SDL_KEYDOWN, KMOD_RCTRL, 0)
      layer_manager_:insertEvent(up_key_event_)

      down_key_event_ = CallbackEvent()
      down_key_event_:key(SDLK_n, SDL_KEYDOWN, KMOD_LCTRL, 0)
      down_key_event_:key(SDLK_n, SDL_KEYDOWN, KMOD_RCTRL, 0)
      layer_manager_:insertEvent(down_key_event_)

      -- 終了キーの登録
      quit_event_ = CallbackEvent()
      quit_event_:key(SDLK_q, SDL_KEYDOWN, KMOD_LCTRL, 0)
      quit_event_:key(SDLK_q, SDL_KEYDOWN, KMOD_RCTRL, 0)
      quit_event_:key(SDLK_F4, SDL_KEYDOWN, KMOD_LALT, 0)
      quit_event_:key(SDLK_F4, SDL_KEYDOWN, KMOD_RALT, 0)
      layer_manager_:insertEvent(quit_event_)

      -- 入力の処理
      text_input_ = TextInput()
      layer_manager_:insertEvent(text_input_)

      -- スケジューラの生成
      scheduler_ = Scheduler:new()
   end


startup["Execute"] =
   function(state)
      require("first_menu")
      state:stateMachine():changeState(first_menu)
   end


startup["Exit"] =
   function(state)
   end
