/*!
  \file
  \brief おぼえる、百人一首タイピング

  \author Satofumi KAMIMURA

  $Id: hyakutyping.cpp,v ea1a3e6caa29 2011/01/12 18:05:27 Satofumi $
*/

#include "hyakutyping.h"
#include "luabindTyping.h"
#include "lua_runStateMachine.h"
#include "luabindInit.h"
#include "luabindAudio.h"
#include "luabindGui.h"
#include "luabindGeometry.h"
#include "luabindCommon.h"
#include "luabindInput.h"
#include "luabindSystem.h"
#include "LuaHandler.h"
#include "Screen.h"
#include "fileExist.h"
#include "Audio.h"
#include "BackMusic.h"
#include "SoundEffect.h"
#include <iostream>
#include <cstring>
#include <cstdlib>

using namespace qrk;
using namespace std;


namespace
{
#ifndef MailAddress
    const char* MailAddress = "satofumi@users.sourceforge.jp";
#endif
#ifndef ProjectPage
    const char* ProjectPage = "http://hyakutyping.sourcefoge.jp/2nd/";
#endif


    class Args
    {
    public:
        bool is_fullscreen;
        bool is_mute;


        Args(void) : is_fullscreen(false), is_mute(false)
        {
        }
    };


    void displayVersion(void)
    {
        cout << "hyakutyping " << VERSION << endl;
    }


    void displayHelp(void)
    {
        cout << "Usage: hyakutyping [options]" << endl
             << endl
             << "options" << endl
             << "  -f, --fullscreen    fullscreen play" << endl
             << "  -m, --mute       mute sound" << endl
             << "  -h, --help       display this help and exit" << endl
             << "  -v, --version    output version information and exit" << endl
             << endl
             << "hyakutyping project page: " << ProjectPage << endl
             << "Report is bugs to " << MailAddress << endl;
    }


    Args parseArgs(int argc, char *argv[])
    {
        Args args;

        bool display_help = false;
        bool display_version = false;
        for (int i = 1; i < argc; ++i) {
            const char* token = argv[i];

            if ((! strcmp(token, "-v")) || (! strcmp(token, "--version"))) {
                display_version = true;
            } else if ((! strcmp(token, "-f")) ||
                       (! strcmp(token, "--fullscreen"))) {
                args.is_fullscreen = true;
            } else if ((! strcmp(token, "-m")) || (! strcmp(token, "--mute"))) {
                args.is_mute = true;
            } else {
                display_help = true;
            }
        }
        if (display_help) {
            displayHelp();
            exit(0);

        } else if (display_version) {
            displayVersion();
            exit(0);
        }

        return args;
    }


    bool initializeScreen(const Args& args)
    {
        Screen screen;

        string caption = string("hyakutyping: ") + version();
        screen.setCaption(caption.c_str(), caption.c_str());

        if (args.is_fullscreen) {
            screen.setFullscreen();
        }
        if (! screen.show(Screen::OpenGL)) {
            cout << "Screen: " << screen.what() << endl;
            return false;
        }

        return true;
    }


    void initializeAudio(bool is_mute)
    {
        Audio audio;
        audio.initialize();
        if (is_mute) {
            BackMusic back_music;
            back_music.setMute(true);

            SoundEffect sound_effect("");
            sound_effect.setMute(true);
        }
    }


    void initializeLua(lua_State* L)
    {
        LuaHandler::addPath(L, "scripts/?.out;scripts/?.lua");

        luabindSystem(L);
        luabindCommon(L);
        luabindInput(L);
        luabindGeometry(L);
        luabindGui(L);
        luabindAudio(L);
        luabindTyping(L);

        // デフォルト設定の読み出し
        if (LuaHandler::dofile_noext(L, "scripts/save") ||
            LuaHandler::dofile_noext(L, "scripts/option_save") ||
            LuaHandler::dofile_noext(L, "scripts/option_default")) {
            exit(1);
        }

        // 設定の読み出し
        const char* load_files[] = {
            "savedata/option_data.lua",
            "savedata/client_id.lua",
        };
        size_t n = sizeof(load_files) / sizeof(load_files[0]);
        for (size_t i = 0; i < n; ++i) {
            const char* file = load_files[i];
            if (fileExist(file)) {
                LuaHandler::dofile_noext(L, file);
            }
        }
    }
}


const char* version(void)
{
    return VERSION;
}


//! main
int main(int argc, char *argv[])
{
    // 引数の解析
    Args args = parseArgs(argc, argv);

    // 画面の初期化
    if (! initializeScreen(args)) {
        exit(1);
    }

    // 音楽デバイスの初期化
    initializeAudio(args.is_mute);

    // Lua スクリプトの起動
    lua_State* L = luabindInit();
    initializeLua(L);
    LuaHandler::dofile_noext(L, "scripts/startup");
    lua_runStateMachine(L, "startup");

    return 0;
}
