-- 最初のメニュー
--
-- Satofumi KAMIMURA
-- $Id: first_menu.lua,v 6e2476dbf9c6 2011/05/15 12:36:33 satofumi $


-- state: 最初のメニュー
first_menu = {}


local version_ = nil
local version_label_ = nil
local title_ = nil
local title_label_ = nil
local menu_ = nil
local surfaces_ = nil
local buttons_ = nil
local fade_completed_ = false
local menu_placed_ = false

local selected_index_ = -1

local scheduler_fadein_menu_id_ = nil
local scheduler_fadein_id_ = nil


function first_menu_fadein()
   if layer_:alpha() < 1.0 then
      local n = 12
      for i = 0, n do
         layer_:setAlpha(i / n)
         Scheduler:yield()
      end
   end
   fade_completed_ = true
end


function first_menu_fadein_menu()
   if menu_:alpha() < 1.0 then
      enter_event_:setEventAcceptable(true)
      menu_:setEventAcceptable(true)

      local n = 12
      for i = 0, n do
         menu_:setAlpha(i / n)
         Scheduler:yield()
      end
   end
end


first_menu["Enter"] =
   function(state)
      -- バージョン情報の作成
      font_:setPixelSize(20)
      version_ = TextSurface(font_, "version: " .. version())
      version_label_ = Label(version_)
      version_label_:setPosition(Point(right(version_label_,
                                             screen_:rect().w) - 8, 8))
      layer_:push_front(version_label_)

      -- タイトルの作成
      font_:setPixelSize(40)
      title_ = TextSurface(font_, "おぼえる 百人一首タイピング")
      title_label_ = Label(title_)
      title_label_:setPosition(Point(center(title_label_,
                                            screen_:rect().w / 2), title_y_))
      layer_:push_front(title_label_)

      -- メニュー項目の作成
      font_:setPixelSize(32)
      menu_ = Menu(screen_:rect(), Menu.Center, Menu.Verbatim)
      local buttons_text = {
         "タイピング練習",
         "履歴の閲覧",
         "オプション",
         "終了",
      }
      local base_rect = Rect(0, 0, 280, 42)
      require("menu_utils")
      buttons_ = {}
      surfaces_ = {}
      createMenu(menu_, buttons_text,
                 base_rect, font_, surfaces_, buttons_)
      menu_:setPosition(Point(center(menu_, screen_:rect().w / 2), menu_y_))
      local alpha = layer_:alpha()
      menu_:setAlpha(alpha)
      if alpha < 1.0 then
         menu_:setEventAcceptable(false)
         enter_event_:setEventAcceptable(false)
      end

      if menu_placed_ then
         layer_:push_front(menu_)
      end
      if selected_index_ > 0 then
         menu_:setIndex(selected_index_ - 1)
      end

      scheduler_fadein_id_ = scheduler_:insert(first_menu_fadein)
   end


first_menu["Execute"] =
   function(state)
      -- 曲の再生を開始する
      if not back_music_:isPlaying() then
         back_music_:play(Music_menu_, 300, BackMusic_Infinity, 1000)
      end

      -- フェードイン
      scheduler_:execute()
      if fade_completed_ and (not menu_placed_) then
         layer_:push_front(menu_)
         menu_placed_ = true
         scheduler_fadein_menu_id_ = scheduler_:insert(first_menu_fadein_menu)
      end

      redraw()
      if menu_placed_ then
         expandMenu(menu_)
      end

      -- メニューで選択された状態へ遷移させる
      if menu_:isClicked() then
      --if true then
         selected_index_ = menu_:clickedId()
         --selected_index_ = 3

         if selected_index_ ~= 4 then
            sound_effect_input_terminated_:play(0.0, 0.0, 0.1)
         end

         if selected_index_ == 1 then
            require("practice_menu")
            state:stateMachine():changeState(practice_menu)

         elseif selected_index_ == 2 then
            require("recent_history")
            state:stateMachine():changeState(recent_history)

         elseif selected_index_ == 3 then
            require("option_config")
            state:stateMachine():changeState(option_config)

         elseif selected_index_ == 4 then
            require("tear_down")
            state:stateMachine():changeState(tear_down)
         end
      end

      if escape_event_:isActive() then
         -- 即座に終了させる
         quit_event_:setActive()
         checkQuit()
      end
   end


first_menu["Exit"] =
   function(state)
      layer_:remove(version_label_)
      layer_:remove(title_label_)
      layer_:remove(menu_)

      scheduler_:remove(scheduler_fadein_menu_id_)
      scheduler_:remove(scheduler_fadein_id_)
      scheduler_fadein_menu_id_ = nil
      scheduler_fadein_id_ = nil

      buttons_ = nil
      surfaces_ = nil
      menu_ = nil
      title_label_ = nil
      title_ = nil
      version_label_ = nil
      version_ = nil

      layer_:setAlpha(1.0)

      collectgarbage("collect")
  end
