-- 決まり字の練習
--
-- Satofumi KAMIMURA
-- $Id: kimariji_typing.lua,v 3d779acf5d87 2010/05/22 15:22:51 Satofumi $

require("input_handler")
require("display_waka")
require("typing_data")
require("display_result_practice")


-- state: 決まり字の練習
kimariji_typing = {}


local DisplaySubTitle = 0
local DisplayWaka = 1
local DisplayResult = 2

local state_ = 0
local waka_index_ = 0
local waka_id_ = 1
local display_percent_ = 0
local stop_watch_ = nil
local start_sec_ = 0.0
local converted_text_ = ""
local input_completed_ = false
local kimariji_input_sample_text_ = nil

local sound_effect_prologue_ = nil


function kimariji_typing_handle_escape(state)

   -- 入力文字があるときの ESC では、入力文字をクリアするだけにする
   if #converted_text_ > 0 then
      converted_text_ = ""
   else
      require("practice_menu")
      back_music_:stop(300)
      state:stateMachine():changeState(practice_menu)
      kimariji_typing_clearState()
   end
end


function kimariji_typing_clearState()
   display_waka_clear()
   display_result_practice_clear()
   waka_layer_:clear()

   state_ = 0
   waka_index_ = 0

   stop_watch_ = nil
   start_sec_ = nil
   converted_text_ = nil
   input_completed_ = false
   kimariji_input_sample_text_ = nil
end


kimariji_typing["Enter"] =
   function(state)
      if state_ == DisplaySubTitle then
         -- サブタイトルの作成
         state:stateMachine():changeState(display_subtitle)
         state_ = DisplayWaka

         -- 曲の再生を停止する
         back_music_:stop(300)

         -- サブタイトル用の曲を再生
         sound_effect_prologue_ = SoundEffect(Sound_prologue_)
         sound_effect_prologue_:play(0.0, 0.0, 0.1)

      elseif state_ == DisplayWaka then
         if sound_effect_prologue_ ~= nil then
            sound_effect_prologue_:stop()
         end
         text_input_:clear()
         display_percent_ = 0
         if waka_index_ == 0 then
            typing_data_clear(TypingData_Kimariji)
            stop_watch_ = StopWatch()

            -- 曲の再生を開始する
            back_music_:play(Music_typing_, 100, BackMusic_Infinity, 100)
         end

         start_sec_ = 0.0
         waka_index_ = waka_index_ + 1
         converted_text_ = ""
         bs_event_:isActive()
         display_waka_remove_left()

         if waka_index_ > #waka_id_array_ then
            state_ = DisplayResult
         else
            waka_id_ = waka_id_array_[waka_index_]
            input_completed_ = false
            typing_data_set_waka_id(waka_id_)
            kimariji_input_sample_text_ =
               waka_handler_kimariji_input_sample(waka_id_)

            display_waka(waka_id_, 0, true)
            display_waka_yomibito(waka_id_, 0)
            display_waka_kimariji(waka_id_)
         end
         stop_watch_:stop()
         stop_watch_:start()
      end
   end


kimariji_typing["Execute"] =
   function(state)
      scheduler_:execute()
      redraw()

      if state_ == DisplayWaka then
         -- 和歌の入力中
         display_percent_ = 100.0 * stop_watch_:ticks() / YomiageSecond

         --input_completed_ = true
         if not input_completed_ then
            -- 和歌の入力が完了していない
            local view_ku = display_waka(waka_id_, display_percent_, true)
            local yomibito_percent = (display_percent_ - 100) * 5
            display_waka_yomibito(waka_id_, yomibito_percent)

            if #converted_text_ == 0 then
               start_sec_ = stop_watch_:ticks()
            end

            -- 入力文字列の変換
            local input = text_input_:text()
            text_input_:clear()
            local expected_text = kimariji_input_sample_text_
            local bs_pressed = false
            converted_text_ =
               input_handler_convert_ignore_miss(converted_text_, input,
                                                 expected_text)

            -- 入力フォームの表示
            display_waka_input(converted_text_)
            display_waka_kimariji_input_sample(kimariji_input_sample_text_,
                                               view_ku)

            if #converted_text_ > 0 and converted_text_ == expected_text then
               input_completed_ = true
               typing_data_lf(start_sec_ / 1000.0,
                              stop_watch_:ticks() / 1000.0)

               -- 入力フォームを消す
               display_waka_remove_inputs()
            end

         else
            -- 入力が完了した
            display_waka_left(waka_index_)
            display_waka(waka_id_, 100, true)
            display_waka_yomibito(waka_id_, 100)

            if waka_index_ >= #waka_id_array_ then
               -- 全ての和歌を入力したら、曲の再生を止める
               back_music_:stop(1000)
            end

            -- 入力完了の効果音を鳴らす
            sound_effect_input_terminated_:play(0.0, 0.0, 0.1)

            -- 改行を待つ
            require("wait_enter")
            state:stateMachine():changeState(wait_enter)
         end

      elseif escape_event_:isActive() then
         -- wait_enter 中に ESC が押されたときは、結果を表示せずに戻るように
         -- ESC の押下を評価する
         kimariji_typing_handle_escape(state)

      elseif state_ == DisplayResult then
         kimariji_typing_clearState()

         -- 結果の表示
         display_result_practice_set_mode(Result_Kimariji)
         display_result_practice_set_result(typing_data_average())
         state:stateMachine():changeState(display_result_practice)
      end

      if escape_event_:isActive() then
         kimariji_typing_handle_escape(state)
      end
   end


kimariji_typing["Exit"] =
   function(state)
      -- 曲の再生を止める
      sound_effect_prologue_ = nil

      collectgarbage("collect")
   end
