-- タイピング・データの記録
--
-- Satofumi KAMIMURA
-- $Id$


_G.TypingData_Shimonoku = 0
_G.TypingData_Kimariji = 1
_G.TypingData_Network = 2

_G.TypingData_ESC = -1
_G.TypingData_BS = -2
_G.TypingData_Enter = -3


local data_ = {}
local waka_id_ = {}
local size_ = 0
local mode_ = nil
local start_sec_ = {}
local end_sec_ = {}


-- 記録情報のクリア
function typing_data_clear(mode)
   data_ = { {} }
   waka_id_ = {}
   start_sec_ = {}
   end_sec_ = {}
   size_ = 0
   mode = mode
end


-- 和歌の ID を登録
function typing_data_set_waka_id(id)
   waka_id_[size_ + 1] = id
end


-- タイプ文字の追加
function typing_data_add(key, timestamp)
   table.insert(data_[size_ + 1], {key = key, timestamp = timestamp})
end


-- タイプ文字列の追加
function typing_data_add_string(input, timestamp)
   for i = 1, #input do
      table.insert(data_[size_ + 1],
                   {key = input:sub(i, i), timestamp = timestamp})
   end
end


-- １首の入力完了
function typing_data_lf(start_sec, end_sec)
   table.insert(start_sec_, start_sec)
   table.insert(end_sec_, end_sec)
   table.insert(data_, {})
   size_ = size_ + 1
end


-- 記録情報の平均値を取得
function typing_data_average()

   local types = 0
   local erase_types = 0

   local start_sec_average = 0.0
   local end_sec_average = 0.0

   for id, waka in pairs(data_) do
      if #waka == 0 then
         break
      end

      for index, type in pairs(waka) do

         if type.key == TypingData_BS or
            type.key == TypingData_ESC then
            -- BS, ESC の数をカウントする
            erase_types = erase_types + 1
         else
            -- 通常のキー入力の数をカウントする
            types = types + 1
         end
      end
   end

   for i = 1, #start_sec_ do
      start_sec_average = start_sec_average + start_sec_[i]
      end_sec_average = end_sec_average + end_sec_[i]
   end

   start_sec_average = start_sec_average / size_
   end_sec_average = end_sec_average / size_
   local miss_types = erase_types / size_

   if mode_ == TypingData_Kimariji then
      -- 決まり字タイピング
      -- !!!
      return start_sec_average, 0.0, 0.0
   else
      -- 下の句タイピング
      return start_sec_average, end_sec_average, miss_types
   end
end


-- 入力の開始時間を返す
function typing_data_start_sec()
   local data = {}
   for k, v in pairs(start_sec_) do
      local id = waka_id_[k]
      local start_sec = start_sec_[k]
      table.insert(data, {id = id, start_sec = start_sec})
   end

   return data
end


-- 記録情報を文字列に変換
function typing_data_string()

   local encoded = ""

   for id, waka in pairs(data_) do
      if #waka == 0 then
         break
      end

      encoded = encoded .. waka_id_[id] .. ", "
      for index, type in pairs(waka) do
         encoded = encoded .. type.key .. "," .. type.timestamp .. ","
      end
      encoded = encoded .. "0,0,"
   end

   return encoded
end
