# -*- coding: utf-8 -*-
# !/usr/bin/ruby
# 状態遷移を dot 形式にパースするスクリプト
# Satofumi KAMIMURA
# $Id$

# キーは、状態のファイル名
$next_flows = {}                 # 値は、次の状態のファイル名への配列
$previous_flows = {}             # 値は、次の状態のファイル名への配列
$titles = {}                     # 値は、表示用の状態名
$state_order = []                # ファイル処理順の記録用
$ignore_state = {}               # 処理の対象にしない状態名


def parseFile(file)

  File.open(file) { |io|
    # 状態名の生成
    state_name = File.basename(file, ".*")
    $state_order.push(state_name)

    # フロー、表示用の状態名の取得
    while line = io.gets
      case line.chomp

      when /\s*-- state: (.+?)\s*$/
        # 日本語での状態名を取得
        title = $1
        $titles[state_name] = title
        $next_flows[state_name] = []

      when /\s*--/
        # Lua のコメントを読み飛ばす
        # !!! 複数行のコメント形式にも対応すべき

      when /:stateMachine\(\):changeState\((.+)\)/
        # 分岐の抽出
        next_state = $1
	if $next_flows[state_name] then
          $next_flows[state_name].push(next_state)
	end

      when /:stateMachine\(\):changePreviousState\(\)/
        # 戻る分岐の抽出
        $previous_flows[state_name] = []
      end
    end
  }
end



# 引数が渡されなければ、使い方を表示して終了
if ARGV.empty? then
  print "usage\n\t" + __FILE__ + " [-o output] <lua files> [-o ignore files]\n"
  exit(1)
end

# 出力ファイルの決定
output_file = "states.dot"
if (ARGV.size >= 2) && (ARGV[0] == "-o")
  output_file = ARGV[1]
  ARGV.shift
  ARGV.shift
end

# 無視するファイルの取得
ignore_flag = false
ignore_files = []
parse_files = []
ARGV.each { |file|
  if file == "-i" then
    ignore_flag = true
    next
  end
  if ignore_flag then
    ignore_files.push(file)
  else
    parse_files.push(file)
  end
}


# 無視する状態名の読み出し
ignore_files.each { |file|
  state_name = File.basename(file, ".*")
  $ignore_state[state_name] = file
}


# 状態の読み出し
parse_files.each { |file|
  parseFile(file)
}


# パース結果を DOT 形式に変換して出力
File.open(output_file, "w") { |io|
  io.print("digraph chart {\n")

  $state_order.each { |state_name|

    # 状態毎に情報を出力
    io.print("\n")

    title = $titles[state_name]
    if title then
      io.print("  " + state_name + " [label=\"" + title + "\\n(" + state_name + ")\", fontname=\"default\", fontsize=11];\n")
    end

    # フローを出力
    if $next_flows[state_name] then
      $next_flows[state_name].each { |next_state|
        if $ignore_state.has_key?(next_state) then
          next
        end
        io.print("  " + state_name + " -> " + next_state + "\n")

        # 戻るフローの登録
        if $previous_flows.key?(next_state) then
          $previous_flows[next_state].push(state_name)
        end
      }
    end
  }

  # 戻るフローの作成
  $previous_flows.each { |state_name, next_states|
    next_states.each { |next_state|
      if $ignore_state.has_key?(next_state) then
        next
      end
      io.print("  " + state_name + " -> " + next_state + " [style = dotted];\n")
    }
  }

  io.print("}\n")
}
